@@ -1,22 +1,9 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * LUKS - Linux Unified Key Setup v2, PBKDF2 digest handler (LUKS1 compatible)
  *
- * Copyright (C) 2015-2021 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2015-2021 Milan Broz
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * Copyright (C) 2015-2025 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2015-2025 Milan Broz
  */
 
 #include "luks2_internal.h"
@@ -33,10 +20,10 @@ static int PBKDF2_digest_verify(struct crypt_device *cd,
 	char checkHashBuf[64];
 	json_object *jobj_digest, *jobj1;
 	const char *hashSpec;
-	char *mkDigest = NULL, mkDigestSalt[LUKS_SALTSIZE];
+	char *mkDigest = NULL, *mkDigestSalt = NULL;
 	unsigned int mkDigestIterations;
 	size_t len;
-	int r;
+	int r = -EINVAL;
 
 	/* This can be done only for internally linked digests */
 	jobj_digest = LUKS2_get_digest_jobj(crypt_get_hdr(cd, CRYPT_LUKS2), digest);
@@ -53,25 +40,23 @@ static int PBKDF2_digest_verify(struct crypt_device *cd,
 
 	if (!json_object_object_get_ex(jobj_digest, "salt", &jobj1))
 		return -EINVAL;
-	len = sizeof(mkDigestSalt);
-	if (!base64_decode(json_object_get_string(jobj1),
-			   json_object_get_string_len(jobj1), mkDigestSalt, &len))
-		return -EINVAL;
+	r = crypt_base64_decode(&mkDigestSalt, &len, json_object_get_string(jobj1),
+				json_object_get_string_len(jobj1));
+	if (r < 0)
+		goto out;
 	if (len != LUKS_SALTSIZE)
-		return -EINVAL;
+		goto out;
 
 	if (!json_object_object_get_ex(jobj_digest, "digest", &jobj1))
-		return -EINVAL;
-	len = 0;
-	if (!base64_decode_alloc(json_object_get_string(jobj1),
-			   json_object_get_string_len(jobj1), &mkDigest, &len))
-		return -EINVAL;
+		goto out;
+	r = crypt_base64_decode(&mkDigest, &len, json_object_get_string(jobj1),
+				json_object_get_string_len(jobj1));
+	if (r < 0)
+		goto out;
 	if (len < LUKS_DIGESTSIZE ||
 	    len > sizeof(checkHashBuf) ||
-	    (len != LUKS_DIGESTSIZE && len != (size_t)crypt_hash_size(hashSpec))) {
-		free(mkDigest);
-		return -EINVAL;
-	}
+	    (len != LUKS_DIGESTSIZE && len != (size_t)crypt_hash_size(hashSpec)))
+		goto out;
 
 	r = -EPERM;
 	if (crypt_pbkdf(CRYPT_KDF_PBKDF2, hashSpec, volume_key, volume_key_len,
@@ -80,11 +65,12 @@ static int PBKDF2_digest_verify(struct crypt_device *cd,
 			mkDigestIterations, 0, 0) < 0) {
 		r = -EINVAL;
 	} else {
-		if (memcmp(checkHashBuf, mkDigest, len) == 0)
+		if (crypt_backend_memeq(checkHashBuf, mkDigest, len) == 0)
 			r = 0;
 	}
-
+out:
 	free(mkDigest);
+	free(mkDigestSalt);
 	return r;
 }
 
@@ -148,30 +134,38 @@ static int PBKDF2_digest_store(struct crypt_device *cd,
 		json_object_object_get_ex(hdr->jobj, "digests", &jobj_digests);
 	}
 
+	if (!jobj_digest)
+		return -ENOMEM;
+
 	json_object_object_add(jobj_digest, "type", json_object_new_string("pbkdf2"));
 	json_object_object_add(jobj_digest, "keyslots", json_object_new_array());
 	json_object_object_add(jobj_digest, "segments", json_object_new_array());
 	json_object_object_add(jobj_digest, "hash", json_object_new_string(pbkdf.hash));
 	json_object_object_add(jobj_digest, "iterations", json_object_new_int(pbkdf.iterations));
 
-	base64_encode_alloc(salt, LUKS_SALTSIZE, &base64_str);
-	if (!base64_str) {
+	r = crypt_base64_encode(&base64_str, NULL, salt, LUKS_SALTSIZE);
+	if (r < 0) {
 		json_object_put(jobj_digest);
-		return -ENOMEM;
+		return r;
 	}
 	json_object_object_add(jobj_digest, "salt", json_object_new_string(base64_str));
 	free(base64_str);
 
-	base64_encode_alloc(digest_raw, hmac_size, &base64_str);
-	if (!base64_str) {
+	r = crypt_base64_encode(&base64_str, NULL, digest_raw, hmac_size);
+	if (r < 0) {
 		json_object_put(jobj_digest);
-		return -ENOMEM;
+		return r;
 	}
 	json_object_object_add(jobj_digest, "digest", json_object_new_string(base64_str));
 	free(base64_str);
 
-	if (jobj_digests)
-		json_object_object_add_by_uint(jobj_digests, digest, jobj_digest);
+	if (jobj_digests) {
+		r = json_object_object_add_by_uint(jobj_digests, digest, jobj_digest);
+		if (r < 0) {
+			json_object_put(jobj_digest);
+			return r;
+		}
+	}
 
 	JSON_DBG(cd, jobj_digest, "Digest JSON:");
 	return 0;
