@@ -1,24 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * cryptsetup - setup cryptographic volumes for dm-crypt
  *
  * Copyright (C) 2004 Jana Saout <jana@saout.de>
  * Copyright (C) 2004-2007 Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2024 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2024 Milan Broz
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * Copyright (C) 2009-2025 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2025 Milan Broz
  */
 
 #include "cryptsetup.h"
@@ -438,27 +425,27 @@ void tools_package_version(const char *name, bool use_pwlibs)
 	bool udev = false, blkid = false, keyring = false, fips = false,
 	     kernel_capi = false, pwquality = false, passwdqc = false,
 	     hw_opal = false;
-#ifdef USE_UDEV
+#if USE_UDEV
 	udev = true;
 #endif
-#ifdef HAVE_BLKID
+#if HAVE_BLKID
 	blkid = true;
 #endif
-#ifdef KERNEL_KEYRING
+#if KERNEL_KEYRING
 	keyring = true;
 #endif
-#ifdef ENABLE_FIPS
+#if ENABLE_FIPS
 	fips = true;
 #endif
-#ifdef ENABLE_AF_ALG
+#if ENABLE_AF_ALG
 	kernel_capi = true;
 #endif
-#if defined(ENABLE_PWQUALITY)
+#if ENABLE_PWQUALITY
 	pwquality = true;
-#elif defined(ENABLE_PASSWDQC)
+#elif ENABLE_PASSWDQC
 	passwdqc = true;
 #endif
-#ifdef HAVE_HW_OPAL
+#if HAVE_HW_OPAL
 	hw_opal = true;
 #endif
 	log_std("%s %s flags: %s%s%s%s%s%s%s%s\n", name, PACKAGE_VERSION,
@@ -471,3 +458,19 @@ void tools_package_version(const char *name, bool use_pwlibs)
 		passwdqc && use_pwlibs ? "PASSWDQC " : "",
 		hw_opal ? "HW_OPAL " : "");
 }
+
+int tools_check_newname(const char *name)
+{
+	if (!name)
+		return 0;
+
+	if (strlen(name) >= DM_NAME_LEN) {
+		log_err(_("Name is too long."));
+		return -EINVAL;
+	} else if (strchr(name, '/')) {
+		log_err(_("Name must not contain '/' character."));
+		return -EINVAL;
+	}
+
+	return 0;
+}
